/*-----------------------------------------------------------------------

File  : che_proofcontrol.c

Author: Stephan Schulz

Contents
 
  Basic functions for proof control objects.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Oct 16 16:20:00 MET DST 1998
    Salvaged from cco_proofstate.h, forked control and state.
<2> Wed Dec 16 18:45:14 MET 1998
    Moved from cco to che

-----------------------------------------------------------------------*/

#include "che_proofcontrol.h"


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

char* DefaultWeightFunctions = 
"\n"
"weight21_ugg  = Clauseweight(PreferUnitGroundGoals,2,1,1)      \n"
"rweight21_g   = Refinedweight(PreferGoals,    2,1,1.5,1.1,1.1) \n"
"rweight21_a   = Refinedweight(PreferNonGoals, 2,1,1.5,1.1,1.1) \n"
"rweight21_ugg = Refinedweight(PreferUnitGroundGoals, 2,1,1.5,1.1,1.1) \n"
"fifo_f        = FIFOWeight(ConstPrio)                      \n"
"lifo_f        = LIFOWeight(ConstPrio)                      \n"
"weight11_f    = Clauseweight(ConstPrio,1,1,1)              \n"
"weight21_f    = Clauseweight(ConstPrio,2,1,1)              \n"
"/* Here start the auto-mode-weight functions */            \n"
"auto1     = Refinedweight(ConstPrio,1,1,2,2,2)         \n"
"auto2     = Refinedweight(PreferGoals,1,2,1.5,1,1)     \n"
"auto3     = Refinedweight(PreferNonGoals,2,1,1.5,1,1)  \n"
"auto4     = Refinedweight(PreferGoals,1,2,2,2,2)  \n"
"auto5     = Refinedweight(PreferNonGoals,2,1,2,2,0.5)  \n"
"auto6     = Clauseweight(PreferUnitGroundGoals,1,1,1)  \n"
"auto7     = Sigweight(PreferGoals,1,2,2,2,1,-0.3)  \n"
"auto8     = Sigweight(PreferNonGoals,2,1,2,2,1,-0.3)  \n"
"auto9     = Refinedweight(PreferGoals,1,1,2,1.5,2)  \n"
"auto10    = Refinedweight(PreferNonGoals,1,1,2,1.5,0.5)  \n"
"auto11    = Clauseweight(ConstPrio,1,1,2)  \n"
"auto13    = Clauseweight(ConstPrio,1,1,1.5)  \n"
"auto14    = Refinedweight(ConstPrio,1,1,2,1.4,1)  \n"
"auto15    = Refinedweight(PreferGoals,1,1,2,1.2,2)  \n"
"auto18    = FIFOWeight(PreferUnitGroundGoals)  \n"
"auto19    = Refinedweight(ConstPrio,2,1,2,1,1)  \n"
"auto20    = Refinedweight(PreferGoals,2,1,3.5,1,1)  \n"
"auto21    = Clauseweight(ConstPrio,2,3,1)  \n"
"auto22    = Sigweight(PreferGoals,2,1,3.0,1,1,-0.3)  \n"
"auto23    = Clauseweight(ConstPrio,1,1,2)  \n"
"auto24    = Refinedweight(PreferNonGoals,1,2,2,1,1)  \n"
"auto25    = Refinedweight(PreferGoals,1,2,2,1,1)  \n"
"auto26    = Clauseweight(ByGoalDifficulty,1,1,1.5)\n"
"auto27    = Proofweight(ConstPrio,1,1,2,1.2,0.7,0,0.5)\n"
;

char* DefaultHeuristics = 
"Weight     = (1*weight21_ugg)                       \n"
"RWeight    = (1*rweight21_ugg)                      \n"
"FIFO       = (1*fifo_f)                             \n"
"LIFO       = (1*lifo_f)                             \n"
"Default    = (3*rweight21_a, 1*rweight21_g)         \n"
"Uniq       = (1*Uniqweight(ConstPrio))\n"
"/* Here start the auto-mode-heuristics*/            \n"
"H____0a = (8*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"            8*Refinedweight(PreferNonGoals,2,1,2,2,1),\n"
"            1*Clauseweight(ConstPrio,1,1,1),\n"
"            1*FIFOWeight(ConstPrio))\n"
"H____1a = (1*Refinedweight(ConstPrio,1,1,2,2,1))\n"
"H____1b = (1*Refinedweight(ConstPrio,1,1,2,2,1))\n"
"H____3b = (1*Refinedweight(ConstPrio,1,1,2,2,0.5))\n"
"H____4b = (8*Refinedweight(PreferGoals,1,2,2,2,2),\n"
"            8*Refinedweight(PreferNonGoals,2,1,2,2,0.5),\n"
"            1*Clauseweight(ConstPrio,1,1,1),\n"
"            1*FIFOWeight(ConstPrio))\n"
"H____Ba = (8*Refinedweight(PreferGoals,1,2,2,2,2),\n"
"            8*Refinedweight(PreferNonGoals,2,1,2,2,0.5),\n"
"            1*Clauseweight(PreferUnitGroundGoals,1,1,1),\n"
"            1*FIFOWeight(ConstPrio))\n"
"H____Cb = (10*Sigweight(PreferGoals,1,2,2,2,1,-0.3),\n"
"            10*Sigweight(PreferNonGoals,2,1,2,2,1,-0.3),\n"
"            1*Clauseweight(ConstPrio,1,1,1),\n"
"            1*FIFOWeight(ConstPrio))\n"
"H____7a = (1*Refinedweight(PreferGoals,1,2,1.5,1,1),\n"
"            2*Refinedweight(PreferNonGoals,2,1,1.5,1,1))\n"
"H____Eb = (8*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"            8*Refinedweight(PreferNonGoals,2,1,2,2,1),\n"
"            1*Clauseweight(ConstPrio,1,1,1),\n"
"            1*FIFOWeight(ByNegLitDist))\n"

"G_R  = (8*auto14,1*fifo_f)\n"
"G_V  = (3*auto15,3*auto10,1*auto13,1*auto18)\n"
"G_Z  = (3*auto9,3*auto10,1*auto11,1*auto18)\n"
"GN01 = (2*auto9, 3*auto10,2*auto26,1*fifo_f)\n"
"GN05 = (5*auto19, 1*fifo_f)\n"
"GN08 = (6*auto27, 1*fifo_f)\n"

"UG5  = (1*auto19)\n"
"UG_Z = (25*auto20,1*auto21,1*fifo_f)\n"
"UG_H = (10*auto20,1*weight21_f,1*fifo_f)\n"
"UGJ  = (25*auto22,5*weight11_f,1*fifo_f)\n"
"UNU  = (1*auto23)\n"
"UN_K = (2*auto24,8*auto25,1*fifo_f)\n"
;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: ProofControlFree()
//
//   Allocate an empty, initialized ProofControlCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

ProofControl_p ProofControlAlloc()
{
   ProofControl_p handle = ProofControlCellAlloc();
   handle->ocb                     = NULL;
   handle->wfcbs                   = WFCBAdminAlloc();
   handle->hcbs                    = HCBAdminAlloc();
   handle->hcb                     = NULL;
   handle->prefer_initial_clauses  = false;
   handle->strategy                = PMNormal;

   return handle;
}

/*-----------------------------------------------------------------------
//
// Function: ProofControlFree()
//
//   Free a ProofControlCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void ProofControlFree(ProofControl_p junk)
{
   assert(junk);
   if(junk->ocb)
   {
      OCBFree(junk->ocb);
   }
   WFCBAdminFree(junk->wfcbs);
   HCBAdminFree(junk->hcbs);
   /* hcb is always freed in junk->hcbs */
   ProofControlCellFree(junk);
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/







